from django.conf.urls import patterns, url

from django_auth_policy.forms import (StrictAuthenticationForm,
                                      StrictPasswordChangeForm)


urlpatterns = patterns('',
    url(r'^login/$', 'django_auth_policy.views.login', name='login',
        kwargs={'authentication_form': StrictAuthenticationForm,
                'template_name': 'login.html'}),
    url(r'^logout/$', 'django.contrib.auth.views.logout_then_login',
        name='logout'),
    url(r'^password_change/$', 'django.contrib.auth.views.password_change',
        name='password_change',
        kwargs={'password_change_form': StrictPasswordChangeForm,
                'template_name': 'change_password.html',
                'post_change_redirect': '/',
                }),
    )



import logging
try:
    from collections import OrderedDict
except ImportError:
    # python 2.6 or earlier, use backport
    from ordereddict import OrderedDict

from django import forms
from django.utils.translation import ugettext as _
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm

from django_auth_policy.models import PasswordChange
from django_auth_policy.handlers import (PasswordStrengthPolicyHandler,
                                         AuthenticationPolicyHandler,
                                         PasswordChangePolicyHandler)


logger = logging.getLogger(__name__)


class StrictAuthenticationForm(AuthenticationForm):
    auth_policy = AuthenticationPolicyHandler()
    password_change_policy = PasswordChangePolicyHandler()

    def __init__(self, request, *args, **kwargs):
        """ Make request argument required
        """
        return super(StrictAuthenticationForm, self).__init__(
            request, *args, **kwargs)

    def clean(self):
        remote_addr = self.request.META['REMOTE_ADDR']
        host = self.request.get_host()
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        attempt = self.auth_policy.pre_auth_checks(username, password,
                                                   remote_addr, host)

        self.user_cache = authenticate(username=username, password=password)
        attempt.user = self.user_cache

        self.auth_policy.post_auth_checks(attempt)

        self.password_change_policy.update_session(self.request, self.user_cache)

        return self.cleaned_data


class StrictSetPasswordForm(forms.Form):
    password_strength_policy = PasswordStrengthPolicyHandler()
    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
        }
    new_password1 = forms.CharField(label=_("New password"),
                                    widget=forms.PasswordInput)
    new_password2 = forms.CharField(label=_("New password confirmation"),
                                    widget=forms.PasswordInput)

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(StrictSetPasswordForm, self).__init__(*args, **kwargs)

    def clean_new_password1(self):
        pw = self.cleaned_data.get('new_password1')
        self.password_strength_policy.validate(pw, self.user)
        return pw

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(
                    self.error_messages['password_mismatch'],
                    code='password_mismatch',
                )
        return password2

    def is_valid(self):
        valid = super(StrictSetPasswordForm, self).is_valid()
        if self.is_bound:
            PasswordChange.objects.create(user=self.user, successful=valid,
                                          is_temporary=False)
            if valid:
                logger.info('Password change successful for user %s',
                            self.user)
            else:
                logger.info('Password change failed for user %s',
                            self.user)
        return valid

    def save(self, commit=True):
        self.user.set_password(self.cleaned_data['new_password1'])
        if commit:
            self.user.save()
        return self.user


class StrictPasswordChangeForm(StrictSetPasswordForm):
    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
        'password_incorrect': _("Your old password was entered incorrectly. "
                                "Please enter it again."),
        'password_unchanged': _("The new password must not be the same as "
                                "the old password"),
        }
    old_password = forms.CharField(label=_("Old password"),
                                   widget=forms.PasswordInput)

    def clean_old_password(self):
        """
        Validates that the old_password field is correct.
        """
        old_password = self.cleaned_data["old_password"]
        if not self.user.check_password(old_password):
            raise forms.ValidationError(
                self.error_messages['password_incorrect'],
                code='password_incorrect',
            )
        return old_password

    def clean_new_password1(self):
        pw = super(StrictPasswordChangeForm, self).clean_new_password1()

        # Check that old and new password differ
        if (self.cleaned_data.get('old_password') and
                self.cleaned_data['old_password'] == pw):

            raise forms.ValidationError(
                self.error_messages['password_unchanged'],
                'password_unchanged')

        return pw


StrictPasswordChangeForm.base_fields = OrderedDict(
    (k, StrictPasswordChangeForm.base_fields[k])
    for k in ['old_password', 'new_password1', 'new_password2']
)



import logging

from django.conf import settings
from django.utils.module_loading import import_by_path
from django.db import transaction
from django.core.exceptions import ValidationError

from django_auth_policy.models import (LoginAttempt, PasswordChange)

logger = logging.getLogger(__name__)


class PasswordStrengthPolicyHandler(object):
    """ Runs all policies related to password strength requirements
    Raises a ValidationError when a password doesn't comply
    """
    _policies = []
    policy_texts = []

    def __init__(self):
        if self._policies:
            return

        for policy_path, kwargs in settings.PASSWORD_STRENGTH_POLICIES:
            policy_class = import_by_path(policy_path)
            policy = policy_class(**kwargs)

            self._policies.append(policy)

            if policy.show_policy and policy.policy_text:
                self.policy_texts.append({
                    'text': policy.policy_text,
                    'caption': policy.policy_caption,
                })

    def validate(self, password, user):
        """ Validate password strength against all password policies.
        One should also provide the user that (will) use this password.
        Policies will raise a ValidationError when the password doesn't comply
        """
        for pol in self._policies:
            pol.validate(password, user)


class PasswordChangePolicyHandler(object):
    """ Runs all policies related to enforced password changes
    Raises a ValidationError when a user is enforced to change its password
    """
    _policies = []
    policy_texts = []

    def __init__(self):
        if self._policies:
            return

        for policy_path, kwargs in settings.PASSWORD_CHANGE_POLICIES:
            policy_class = import_by_path(policy_path)
            policy = policy_class(**kwargs)

            self._policies.append(policy)

    def validate(self, user):
        try:
            last_pw_change = PasswordChange.objects.filter(
                user=user, successful=True).order_by('-id')[0]
        except IndexError:
            last_pw_change = None

        for pol in self._policies:
            pol.validate(last_pw_change)

    def update_session(self, request, user):
        if not hasattr(request, 'session'):
            return

        try:
            self.validate(user)
        except ValidationError as exc:
            if request.session.get('password_change_enforce') != exc.code:
                logger.info(u'User %s must change password; %s',
                            user, exc.code)
            request.session['password_change_enforce'] = exc.code
            request.session['password_change_enforce_msg'] = \
                unicode(exc.message)
        else:
            request.session['password_change_enforce'] = False
            request.session['password_change_enforce_msg'] = None


class AuthenticationPolicyHandler(object):
    """ Runs all policies related to authentication
    Raises a ValidationError when an authentication attempt does not comply
    """
    _policies = []
    policy_texts = []

    def __init__(self):
        if self._policies:
            return

        for policy_path, kwargs in settings.AUTHENTICATION_POLICIES:
            policy_class = import_by_path(policy_path)
            policy = policy_class(**kwargs)

            self._policies.append(policy)

    def pre_auth_checks(self, username, password, remote_addr, host):
        """ Policy checks before a user is authenticated
        No `User` instance is available yet

        Raises ValidationError for failed login attempts
        On success it returns a LoginAttempt instance

        `username` must be a string that uniquely identifies a user.
        """
        logger.info('Authentication attempt, username=%s, address=%s',
                    username, remote_addr)

        with transaction.atomic():
            username_len = LoginAttempt._meta.get_field('username').max_length
            hostname_len = LoginAttempt._meta.get_field('hostname').max_length
            attempt = LoginAttempt.objects.create(
                username=username[:username_len] if username else '-',
                source_address=remote_addr,
                hostname=host[:hostname_len],
                successful=False,
                lockout=True)

        for pol in self._policies:
            pol.pre_auth_check(attempt, password)

        return attempt

    def post_auth_checks(self, attempt):
        """ Policy checks after the user has been authenticated.
        The attempt should now have a `user`
        """
        for pol in self._policies:
            pol.post_auth_check(attempt)

        # Authentication was successful
        logger.info(u'Authentication success, username=%s, address=%s',
                    attempt.username, attempt.source_address)

        attempt.successful = True
        attempt.lockout = False
        attempt.save()

        for pol in self._policies:
            pol.auth_success(attempt)

        return attempt



import logging

from django import forms
from django import template
from django import http
from django.conf.urls import patterns, url
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.utils import timezone
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth import get_user_model

from django_auth_policy.models import PasswordChange, LoginAttempt, UserChange
from django_auth_policy import signals


logger = logging.getLogger(__name__)

user_model = get_user_model()


class StrictUserCreationForm(forms.ModelForm):
    """Alternative for Djangos UserCreationForm but doesn't set password
    and allows editing email and name right from the start"""
    error_messages = UserCreationForm.error_messages
    username = forms.RegexField(label=_("Username"), max_length=30,
                                regex=r'^[\w.@+-]+$',
                                help_text=_("Required. 30 characters or "
                                            "fewer. Letters, digits and "
                                            "@/./+/-/_ only."),
                                error_messages={
                                    'invalid': _("This value may contain only "
                                                 "letters, numbers and "
                                                 "@/./+/-/_ characters.")})

    class Meta:
        model = user_model
        fields = ('username', 'email', 'first_name', 'last_name')

    def save(self, commit=True):
        user = super(StrictUserCreationForm, self).save(commit=False)
        if commit:
            user.save()
        return user


class StrictUserAdmin(UserAdmin):
    """ Django user admin that enforces policies of django_auth_policy

    * Removes ability to set passwords of other users;
    * Adds ability to generate a temporary password;
    * Adds ability to unlock usernames / IP addresses;
    * Adds ability to re-activate disabled users;
    * Disables users that expired
    """
    readonly_fields = ('last_login', 'date_joined')
    add_form = StrictUserCreationForm
    add_fieldsets = ((None, {'classes': ('wide',),
                             'fields': ('username', 'email', 'first_name',
                                        'last_name')}),)
    list_display = ('username', 'email', 'is_active', 'last_login', 'is_staff')

    actions = ['reactivate_users', 'unlock_username',
               'temporary_password_action']

    def get_actions(self, request):
        # Disable deletion of users, because this would alter LoginAttempt and
        # PasswordChange history
        actions = super(StrictUserAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def unlock_username(self, request, queryset):
        LoginAttempt.objects.filter(username__in=queryset.values('username'),
                                    lockout=True).update(lockout=False)

        self.message_user(request, _('Unlocked selected usernames'))
    unlock_username.short_description = _('Unlock usernames of users')

    def reactivate_users(self, request, queryset):
        fixed = queryset.filter(is_active=False
                                ).update(is_active=True,
                                         last_login=timezone.now())
        self.message_user(request, _('%s users were (re)activated') % fixed)
    reactivate_users.short_description = _('(Re)activate users')

    def get_urls(self):
        urls = super(StrictUserAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(r'^password/$',
                self.admin_site.admin_view(self.set_temporary_password),
                name='set_temporary_password'),
            url(r'^(\d+)/password/$',
                self.admin_site.admin_view(self.user_change_password))
            )
        return my_urls + urls

    def temporary_password_action(self, request, queryset):
        user_ids = ['pk=' + str(i) for i in queryset.values_list('pk',
                                                                 flat=True)]
        return http.HttpResponseRedirect(
            reverse('admin:set_temporary_password') + '?' + '&'.join(user_ids))
    temporary_password_action.short_description = _('Set temporary password')

    def user_change_password(self, request, user_id):
        if not self.has_change_permission(request):
            raise PermissionDenied
        return http.HttpResponseRedirect(
            reverse('admin:set_temporary_password') + '?pk=' + str(user_id))

    def set_temporary_password(self, request):
        if not self.has_change_permission(request):
            raise PermissionDenied

        if not request.GET.get('pk', False):
            return http.HttpResponseRedirect(
                reverse('admin:auth_user_changelist'))

        users = user_model.objects.filter(pk__in=request.GET.getlist('pk'))
        if request.method == 'POST':
            passwords = []
            for user in users:
                password = PasswordChange.objects.set_temporary_password(user)
                passwords.append((user, password))

                signals.temporary_password_set.send(sender=user, user=user,
                                                    request=request,
                                                    password=password)

            return template.response.TemplateResponse(request,
                'admin/django_auth_policy/set_temporary_password.html',
                {'passwords': passwords})

        return template.response.TemplateResponse(request,
            'admin/django_auth_policy/confirm_temporary_password.html',
            {'for_users': users})

    def save_model(self, request, obj, form, change):
        obj.save()
        UserChange.objects.create(user=obj, by_user=request.user)



import logging

from django.conf import settings
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.utils.http import is_safe_url
from django.shortcuts import resolve_url
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect

# Avoid shadowing the login() view below.
from django.contrib.auth import REDIRECT_FIELD_NAME, login as auth_login
from django.contrib.sites.models import get_current_site

from django_auth_policy.forms import StrictAuthenticationForm


logger = logging.getLogger(__name__)


@sensitive_post_parameters()
@csrf_protect
@never_cache
def login(request, template_name='registration/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=StrictAuthenticationForm,
          current_app=None, extra_context=None):
    """
    Displays the login form and handles the login action.

    This view is the same as the Django 1.6 login view but uses the
    StrictAuthenticationForm
    """
    redirect_to = request.POST.get(redirect_field_name,
                                   request.GET.get(redirect_field_name, ''))

    if request.method == "POST":
        form = authentication_form(request, data=request.POST)
        if form.is_valid():

            # Ensure the user-originating redirection url is safe.
            if not is_safe_url(url=redirect_to, host=request.get_host()):
                redirect_to = resolve_url(settings.LOGIN_REDIRECT_URL)

            # Okay, security check complete. Log the user in.
            auth_login(request, form.get_user())

            return HttpResponseRedirect(redirect_to)
    else:
        form = authentication_form(request)

    current_site = get_current_site(request)

    context = {
        'form': form,
        redirect_field_name: redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    }
    if extra_context is not None:
        context.update(extra_context)
    return TemplateResponse(request, template_name, context,
                            current_app=current_app)



# -*- coding: utf-8 -*-
import datetime
import logging
import collections
from cStringIO import StringIO

from django.test import TestCase, RequestFactory
from django.contrib.auth import get_user_model, SESSION_KEY
from django.core.urlresolvers import reverse
from django.utils import timezone

from django_auth_policy.models import LoginAttempt, PasswordChange
from django_auth_policy.forms import (StrictAuthenticationForm,
                                      StrictPasswordChangeForm,
                                      StrictSetPasswordForm)
from django_auth_policy.authentication import (AuthenticationLockedUsername,
                                               AuthenticationLockedRemoteAddress,
                                               AuthenticationDisableExpiredUsers)
from django_auth_policy.password_change import (PasswordChangeExpired,
                                                PasswordChangeTemporary)
from django_auth_policy.password_strength import (PasswordMinLength,
                                                  PasswordContainsUpperCase,
                                                  PasswordContainsLowerCase,
                                                  PasswordContainsNumbers,
                                                  PasswordContainsSymbols,
                                                  PasswordUserAttrs,
                                                  PasswordDisallowedTerms)


class LoginTests(TestCase):
    urls = 'django_auth_policy.testsite.urls'

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username='rf',
            email='rf@example.rf',
            password='password')

        self.factory = RequestFactory()

        self.logger = logging.getLogger()
        self.old_stream = self.logger.handlers[0].stream
        self.logger.handlers[0].stream = StringIO()

    def tearDown(self):
        self.logger.handlers[0].stream = self.old_stream

    def test_success(self):
        """ Test view with form and successful login """
        resp = self.client.get(reverse('login'))
        self.assertEqual(resp.status_code, 200)

        resp = self.client.post(reverse('login'), data={
            'username': 'rf', 'password': 'password'})
        self.assertEqual(resp.status_code, 302)
        self.assertTrue(SESSION_KEY in self.client.session)
        self.assertEqual(self.client.session[SESSION_KEY], self.user.id)

        attempts = LoginAttempt.objects.filter(username=self.user.username,
                                               successful=True)

        self.assertEqual(attempts.count(), 1)

        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication success, username=rf, address=127.0.0.1\n'
            u'INFO User rf must change password; password-expired\n'))

    def test_username_lockout(self):
        """ Test too many failed login attempts for one username """
        pol = AuthenticationLockedUsername()
        text = unicode(pol.validation_msg)
        for x in xrange(0, pol.max_failed):

            req = self.factory.get(reverse('login'))
            req.META['REMOTE_ADDR'] = '10.0.0.%d' % (x + 1)

            form = StrictAuthenticationForm(request=req, data={
                'username': 'rf', 'password': 'wrong password'})

            self.assertEqual(form.non_field_errors(), [
                form.error_messages['invalid_login'] % {
                    'username': form.username_field.verbose_name}]
            )

        attempts = LoginAttempt.objects.filter(username=self.user.username,
                                               successful=False, lockout=True)

        self.assertEqual(attempts.count(),
                         pol.max_failed)

        # Another failed authentication triggers lockout
        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'wrong password'})
        self.assertEqual(form.non_field_errors(), [text])

        self.assertEqual(attempts.count(),
                         pol.max_failed + 1)

        # Even valid authentication will no longer work now
        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertFalse(form.is_valid())

        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=10.0.0.1\n'
            u'INFO Authentication failure, username=rf, address=10.0.0.1, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf, address=10.0.0.2\n'
            u'INFO Authentication failure, username=rf, address=10.0.0.2, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf, address=10.0.0.3\n'
            u'INFO Authentication failure, username=rf, address=10.0.0.3, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication failure, username=rf, address=127.0.0.1, '
            u'username locked\n'
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication failure, username=rf, address=127.0.0.1, '
            u'username locked\n'))

    def test_address_lockout(self):
        """ Test too many failed login attempts for one address """
        pol = AuthenticationLockedRemoteAddress()
        text = unicode(pol.validation_msg)

        addr = '1.2.3.4'

        for x in xrange(0, pol.max_failed):

            req = self.factory.get(reverse('login'))
            req.META['REMOTE_ADDR'] = addr

            form = StrictAuthenticationForm(request=req, data={
                'username': 'rf%d' % x, 'password': 'wrong password'})

            self.assertEqual(form.non_field_errors(), [
                form.error_messages['invalid_login'] % {
                    'username': form.username_field.verbose_name}]
            )

        attempts = LoginAttempt.objects.filter(source_address=addr,
                                               successful=False, lockout=True)

        self.assertEqual(attempts.count(), pol.max_failed)

        # Another failed authentication triggers lockout
        req = self.factory.get(reverse('login'))
        req.META['REMOTE_ADDR'] = addr
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'wrong password'})
        self.assertEqual(form.non_field_errors(), [text])

        self.assertEqual(attempts.count(), pol.max_failed + 1)

        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf0, address=1.2.3.4\n'
            u'INFO Authentication failure, username=rf0, address=1.2.3.4, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf1, address=1.2.3.4\n'
            u'INFO Authentication failure, username=rf1, address=1.2.3.4, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf2, address=1.2.3.4\n'
            u'INFO Authentication failure, username=rf2, address=1.2.3.4, '
            u'invalid authentication.\n'
            u'INFO Authentication attempt, username=rf, address=1.2.3.4\n'
            u'INFO Authentication failure, username=rf, address=1.2.3.4, '
            u'address locked\n'))

    def test_inactive_user(self):
        self.user.is_active = False
        self.user.save()

        # Valid authentication data, but user is inactive
        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_field_errors(), [
            form.error_messages['invalid_login'] % {
                'username': form.username_field.verbose_name
            }]
        )

        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication failure, username=rf, address=127.0.0.1, '
            u'user inactive.\n'))

    def test_lock_period(self):
        pol = AuthenticationLockedUsername()
        text = unicode(pol.validation_msg)
        for x in xrange(0, pol.max_failed + 1):

            req = self.factory.get(reverse('login'))

            form = StrictAuthenticationForm(request=req, data={
                'username': 'rf', 'password': 'wrong password'})

            self.assertFalse(form.is_valid())

        # User locked out
        self.assertEqual(form.non_field_errors(), [text])

        # Alter timestamps as if they happened longer ago
        period = datetime.timedelta(seconds=pol.lockout_duration)
        expire_at = timezone.now() - period
        LoginAttempt.objects.all().update(timestamp=expire_at)

        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertTrue(form.is_valid())

        # Successful login resets lock count
        locking_attempts = LoginAttempt.objects.filter(lockout=True)
        self.assertEqual(locking_attempts.count(), 0)

    def test_unlock(self):
        """ Resetting lockout data unlocks user """
        pol = AuthenticationLockedUsername()
        text = unicode(pol.validation_msg)
        for x in xrange(0, pol.max_failed + 1):

            req = self.factory.get(reverse('login'))

            form = StrictAuthenticationForm(request=req, data={
                'username': 'rf', 'password': 'wrong password'})

            self.assertFalse(form.is_valid())

        # User locked out
        self.assertEqual(form.non_field_errors(), [text])

        # Unlock user or address
        LoginAttempt.objects.all().update(lockout=False)

        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertTrue(form.is_valid())


class UserExpiryTests(TestCase):
    urls = 'django_auth_policy.testsite.urls'

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username='rf',
            email='rf@example.rf',
            password='password')

        self.factory = RequestFactory()

        self.logger = logging.getLogger()
        self.old_stream = self.logger.handlers[0].stream
        self.logger.handlers[0].stream = StringIO()

    def tearDown(self):
        self.logger.handlers[0].stream = self.old_stream

    def test_expiry(self):
        pol = AuthenticationDisableExpiredUsers()

        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertTrue(form.is_valid())

        # Simulate user didn't log in for a long time
        period = datetime.timedelta(days=pol.inactive_period)
        expire_at = timezone.now() - period
        self.user.last_login = expire_at
        self.user.save()
        LoginAttempt.objects.all().update(timestamp=expire_at)

        # Login attempt disabled user
        req = self.factory.get(reverse('login'))
        form = StrictAuthenticationForm(request=req, data={
            'username': 'rf', 'password': 'password'})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.non_field_errors(), [
            form.error_messages['invalid_login'] % {
                'username': form.username_field.verbose_name
            }]
        )

        # Check log messages
        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication success, username=rf, address=127.0.0.1\n'
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO User rf disabled because last login was at %s\n'
            u'INFO Authentication failure, username=rf, address=127.0.0.1, '
            u'user inactive.\n' % expire_at))


class PasswordChangeTests(TestCase):
    urls = 'django_auth_policy.testsite.urls'

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username='rf',
            email='rf@example.rf',
            password='password')

        self.factory = RequestFactory()

        self.logger = logging.getLogger()
        self.old_stream = self.logger.handlers[0].stream
        self.logger.handlers[0].stream = StringIO()

    def tearDown(self):
        self.logger.handlers[0].stream = self.old_stream

    def test_expiry(self):
        pol = PasswordChangeExpired()

        # Create one recent password change
        pw = PasswordChange.objects.create(user=self.user, successful=True,
                                           is_temporary=False)

        # Redirect to login
        resp = self.client.get(reverse('login_required_view'), follow=True)
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.request['PATH_INFO'], reverse('login'))

        # Login
        resp = self.client.post(reverse('login'), data={
            'username': 'rf', 'password': 'password'}, follow=True)
        self.assertEqual(resp.status_code, 200)
        self.assertTrue(SESSION_KEY in self.client.session)
        self.assertEqual(self.client.session[SESSION_KEY], self.user.id)
        self.assertTrue('password_change_enforce' in self.client.session)
        self.assertFalse(self.client.session['password_change_enforce'])
        self.assertFalse(self.client.session['password_change_enforce_msg'])
        self.assertNotContains(resp, 'new_password1')

        # Test if login worked ok
        resp = self.client.get(reverse('login_required_view'), follow=False)
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.request['PATH_INFO'], '/')

        # Logout
        resp = self.client.get(reverse('logout'), follow=True)
        self.assertFalse(SESSION_KEY in self.client.session)

        # Move PasswordChange into the past
        period = datetime.timedelta(days=pol.max_age)
        expire_at = timezone.now() - period
        pw.timestamp = expire_at
        pw.save()

        # Login will still work
        resp = self.client.post(reverse('login'), data={
            'username': 'rf', 'password': 'password'}, follow=True)
        self.assertEqual(resp.status_code, 200)
        self.assertTrue(SESSION_KEY in self.client.session)
        self.assertEqual(self.client.session[SESSION_KEY], self.user.id)
        self.assertTrue('password_change_enforce' in self.client.session)
        self.assertEqual(self.client.session['password_change_enforce'],
                         'password-expired')
        self.assertEqual(self.client.session['password_change_enforce_msg'],
                         unicode(pol.text))
        self.assertContains(resp, 'old_password')
        self.assertContains(resp, 'new_password1')
        self.assertContains(resp, 'new_password2')

        # And try requesting a different page still displays a change
        # password view
        resp = self.client.get(reverse('another_view'), follow=False)
        self.assertTrue('password_change_enforce' in self.client.session)
        self.assertEqual(self.client.session['password_change_enforce'],
                         'password-expired')
        self.assertContains(resp, 'old_password')
        self.assertContains(resp, 'new_password1')
        self.assertContains(resp, 'new_password2')

        # Post a new password
        resp = self.client.post(reverse('login_required_view'), data={
            'old_password': 'password',
            'new_password1': 'abcABC123!@#',
            'new_password2': 'abcABC123!@#'}, follow=True)
        self.assertFalse(self.client.session['password_change_enforce'])
        self.assertNotContains(resp, 'old_password')
        self.assertNotContains(resp, 'new_password1')
        self.assertNotContains(resp, 'new_password2')
        self.assertEqual(resp.redirect_chain, [('http://testserver/', 302)])

        # Recheck, change password view should be gone
        resp = self.client.get(reverse('login_required_view'), follow=False)
        self.assertNotContains(resp, 'old_password')
        self.assertNotContains(resp, 'new_password1')
        self.assertNotContains(resp, 'new_password2')

        # Logging tests
        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication success, username=rf, address=127.0.0.1\n'
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication success, username=rf, address=127.0.0.1\n'
            u'INFO User rf must change password; password-expired\n'
            u'INFO Password change successful for user rf\n'))

    def test_temporary_password(self):
        pol = PasswordChangeTemporary()
        # Create one recent password change
        PasswordChange.objects.create(user=self.user, successful=True,
                                      is_temporary=True)

        # Login
        resp = self.client.post(reverse('login'), data={
            'username': 'rf', 'password': 'password'})
        self.assertEqual(resp.status_code, 302)
        self.assertTrue(SESSION_KEY in self.client.session)
        self.assertTrue('password_change_enforce' in self.client.session)
        self.assertEqual(self.client.session[SESSION_KEY], self.user.id)
        self.assertEqual(self.client.session['password_change_enforce'],
                         'password-temporary')
        self.assertEqual(self.client.session['password_change_enforce_msg'],
                         unicode(pol.text))

        # Requesting a page shows password change view
        resp = self.client.get(reverse('login_required_view'), follow=True)
        self.assertEqual(resp.request['PATH_INFO'], '/')
        self.assertContains(resp, 'old_password')
        self.assertContains(resp, 'new_password1')
        self.assertContains(resp, 'new_password2')

        # Change the password:
        resp = self.client.post(reverse('login_required_view'), data={
            'old_password': 'password',
            'new_password1': 'A-New-Passw0rd-4-me',
            'new_password2': 'A-New-Passw0rd-4-me'}, follow=True)
        self.assertEqual(resp.redirect_chain, [('http://testserver/', 302)])
        self.assertEqual(resp.request['PATH_INFO'], '/')
        self.assertNotContains(resp, 'old_password')
        self.assertNotContains(resp, 'new_password1')
        self.assertNotContains(resp, 'new_password2')

        self.assertEqual(PasswordChange.objects.all().count(), 2)
        self.assertEqual(PasswordChange.objects.filter(
            is_temporary=True).count(), 1)

        # Logging tests
        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            u'INFO Authentication attempt, username=rf, address=127.0.0.1\n'
            u'INFO Authentication success, username=rf, address=127.0.0.1\n'
            u'INFO User rf must change password; password-temporary\n'
            u'INFO Password change successful for user rf\n'))

    def password_change_login_required(self):
        resp = self.client.post(reverse('password_change'), follow=True)
        self.assertEqual(resp.redirect_chain, [
            ('http://testserver/login/?next=/password_change/', 302)])


class PasswordStrengthTests(TestCase):
    urls = 'django_auth_policy.testsite.urls'

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username='rf',
            email='rf@example.rf',
            password='password')

        self.factory = RequestFactory()

        self.logger = logging.getLogger()
        self.old_stream = self.logger.handlers[0].stream
        self.logger.handlers[0].stream = StringIO()

    def tearDown(self):
        self.logger.handlers[0].stream = self.old_stream

    def test_password_length(self):
        pol = PasswordMinLength()

        new_passwd = 'Aa1.$Bb2.^Cc.Dd5%.Ee6&.Dd7*'
        short_passwd = new_passwd[:pol.min_length]

        # Too short password doesnt work
        form = StrictPasswordChangeForm(self.user, data={
            'old_password': 'password',
            'new_password1': short_passwd[:-1],
            'new_password2': short_passwd[:-1]})

        self.assertFalse(form.is_valid())
        msg = unicode(pol.policy_text)
        self.assertEqual(form.errors['new_password1'], [msg])

        # Longer password does work
        form = StrictPasswordChangeForm(self.user, data={
            'old_password': 'password',
            'new_password1': short_passwd,
            'new_password2': short_passwd})
        self.assertTrue(form.is_valid())

        # Check correct PasswordChange items were created
        self.assertEqual(PasswordChange.objects.all().count(), 2)
        self.assertEqual(PasswordChange.objects.filter(
            successful=True).count(), 1)
        self.assertEqual(PasswordChange.objects.filter(
            successful=False).count(), 1)

        # Logging tests
        self.assertEqual(self.logger.handlers[0].stream.getvalue(), (
            'INFO Password change failed for user rf\n'
            'INFO Password change successful for user rf\n'))

    def test_password_complexity(self):
        # List of policies to check, this must match PasswordContains... items
        # of testsettings.PASSWORD_STRENGTH_POLICIES
        policies = collections.deque([
            PasswordContainsUpperCase(),
            PasswordContainsLowerCase(),
            PasswordContainsNumbers(),
            PasswordContainsSymbols(),
        ])
        for x in xrange(0, len(policies)):
            # Create a password with 4 chars from every policy except one
            passwd = u''.join([pol.chars[:4] for pol in list(policies)[:-1]])
            form = StrictPasswordChangeForm(self.user, data={
                'old_password': 'password',
                'new_password1': passwd,
                'new_password2': passwd})
            failing_policy = policies[-1]
            self.assertFalse(form.is_valid())
            err_msg = unicode(failing_policy.policy_text)
            self.assertEqual(form.errors['new_password1'], [err_msg])

            policies.rotate(1)

    def test_password_differ_old(self):
        """ Make sure new password differs from old password """
        passwd = 'Aa1.$Bb2.^Cc.Dd5%.Ee6&.Dd7*'
        self.user.set_password(passwd)
        self.user.save()

        form = StrictPasswordChangeForm(self.user, data={
            'old_password': passwd,
            'new_password1': passwd,
            'new_password2': passwd})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors['new_password1'],
                         [form.error_messages['password_unchanged']])

    def test_password_user_attrs(self):
        policy = PasswordUserAttrs()

        self.user.username = 'rf'
        self.user.email = 'rf@example.com'
        self.user.first_name = 'Rudolph'
        # Try names with accents:
        self.user.last_name = u'Frögér'
        self.user.save()

        passwds = [
            ('AbcDef#12Rudolph', False),
            ('1234rf#examplE', False),
            ('Rudolph#12345', False),
            # Should match froger with accents:
            (u'Froger#12345', False),
            # Short pieces are allowed, like 'rf':
            ('rf54321#AbCd', True),
        ]
        for passwd, valid in passwds:
            form = StrictSetPasswordForm(self.user, data={
                'new_password1': passwd,
                'new_password2': passwd})
            self.assertEqual(form.is_valid(), valid)
            if not valid:
                self.assertEqual(form.errors['new_password1'],
                                 [unicode(policy.text)])

    def test_password_disallowed_terms(self):
        policy = PasswordDisallowedTerms(terms=['Testsite'])

        passwds = [
            ('123TestSite###', False),
            ('123Site###Test', True),
        ]
        for passwd, valid in passwds:
            form = StrictSetPasswordForm(self.user, data={
                'new_password1': passwd,
                'new_password2': passwd})
            self.assertEqual(form.is_valid(), valid)
            if not valid:
                errs = [unicode(policy.policy_text)]
                self.assertEqual(form.errors['new_password1'], errs)



from django.db import models
from django.conf import settings
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _


class LoginAttemptManager(models.Manager):
    def unlock(self, usernames=[], addresses=[]):
        """ Unlocks given usernames and IP addresses

        Returns the number of attempts that have been unlocked.
        """
        if not usernames and not addresses:
            return 0

        selection = models.Q()

        if usernames:
            selection |= models.Q(username__in=set(usernames))

        if addresses:
            selection |= models.Q(source_address__in=set(addresses))

        return self.get_query_set().filter(selection,
                                           lockout=True).update(lockout=False)

    def unlock_queryset(self, queryset):
        """ Unlocks all usernames and IP addresses found in ``queryset``

        Returns the number of attempts that have been unlocked.
        """
        selected_attempts = queryset.filter(
            lockout=True).order_by().values_list('username', 'source_address')

        if not selected_attempts:
            return 0

        usernames, addresses = zip(*selected_attempts)

        return self.unlock(usernames=usernames, addresses=addresses)


class LoginAttempt(models.Model):
    username = models.CharField(_('username'), max_length=100, db_index=True)
    source_address = models.GenericIPAddressField(
        _('source address'), protocol='both', db_index=True)
    hostname = models.CharField(_('hostname'), max_length=100)
    successful = models.BooleanField(_('successful'), default=False)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True,
                             verbose_name=_('user'), on_delete=models.PROTECT)
    timestamp = models.DateTimeField(_('timestamp'), auto_now_add=True,
                                     default=timezone.now, db_index=True)
    # This is enabled for all failed login attempts. It is reset for every
    # successful login and can be reset by 'user admins'.
    lockout = models.BooleanField(_('lockout'), default=True,
                                  help_text=_('Counts towards lockout count'))

    objects = LoginAttemptManager()

    class Meta:
        verbose_name = _('login attempt')
        verbose_name_plural = _('login attempts')
        ordering = ('-id',)
        permissions = (
            ('unlock', _('Unlock by username or IP address')),
            )

    def __unicode__(self):
        return u'{0} at {1} from {2}'.format(self.username,
                                             self.timestamp,
                                             self.source_address)


class PasswordChangeAdmin(models.Manager):
    def set_temporary_password(self, user):
        """Returns a random password and sets this as temporary password for
        provided user."""
        # Characters used to generate temporary passwords
        allowed_chars = getattr(settings, 'TEMP_PASSWORD_CHARS',
                                'abcdefghijlkmnopqrstuvwxyz'
                                'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
                                '0123456789')
        # Temporary password length
        length = getattr(settings, 'TEMP_PASSWORD_LENGTH', 12)

        password = get_random_string(length, allowed_chars)

        PasswordChange.objects.create(user=user, is_temporary=True,
                                      successful=True)
        user.set_password(password)
        user.save()

        return password


class PasswordChange(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('user'),
                             on_delete=models.PROTECT)
    timestamp = models.DateTimeField(_('timestamp'), auto_now_add=True,
                                     default=timezone.now)
    successful = models.BooleanField(_('successful'), default=False)
    # 'is_temporary is used by 'user managers' to set a temporary password
    # for a user, this password must be changed at first login
    is_temporary = models.BooleanField(_('is temporary'), default=False)
    # Optionally keep password a history of hashes to prevent users from
    # reusing old passwords. FIXME This has *NOT* been implemented
    password = models.CharField(_('password'), max_length=128, default='',
                                editable=False)

    objects = PasswordChangeAdmin()

    class Meta:
        verbose_name = _('password change')
        verbose_name_plural = _('password changes')
        ordering = ('-id',)

    def __unicode__(self):
        return u'{0} at {1}'.format(self.user, self.timestamp)


class UserChange(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('user'),
                             on_delete=models.PROTECT)
    timestamp = models.DateTimeField(_('timestamp'), auto_now_add=True)
    by_user = models.ForeignKey(settings.AUTH_USER_MODEL,
                                verbose_name=_('by user'),
                                related_name='changed_users',
                                on_delete=models.PROTECT)

    class Meta:
        verbose_name = _('user change')
        verbose_name_plural = _('user changes')
        ordering = ('-id',)

    def __unicode__(self):
        return u'{0} at {1} by {1}'.format(self.user, self.timestamp,
                                           self.by_user)
